/************************************************************************
*
*                          etsheadr.h
*
*  Copyright (c) 1992, John N. Sanders, Massachusetts Institute of
*      Technology, Lincoln Laboratory; 244 Wood Street, Lexington,
*      MA 02173.
*  All Rights Reserved.
*  Version 2.6   5/92
*
*             header file describing ETS format data files
*
*************************************************************************
*/

/* Data format header for ETS image data files

Version 0.1;     2/22/90; Original, added frame_type & f_header_length, etc.
Version 0.2;     4/04/90; Added cam_utils & fixed filler1 discrepancy
Version 0.3;     4/11/90; Generalized frame and image concepts
Version 0.4;     8/03/90; Back to v1.2 (sorta) & added fileinfo struct.
Version 1.0;     9/24/91; Extensive revisions by Tennyson, Opar, Lane,
                          & Griffith
Version 1.1;     1/20/93; Task Record added by Tennyson

*/
/*#define SYS_V*/

#ifdef __NeXT__
#define NEED_CPS
#endif
#ifdef sun
#define NEED_CPS
#endif

#ifdef NEED_CPS
#define CLOCKS_PER_SEC 1e6
#endif

/**************** data types & header definition *********************/

#define FFTDAT double
#define int4   unsigned int
#define int2   short int
#define int1   char

#define HEADER_SIZE        4096
#define MAJOR_VER          1
#define MINOR_VER          1
#define IEEE_MACHINE       10
#define NON_IEEE_MACHINE   11
#define UNKNOWN_MACHINE    12
#define PROPER_ORDER       0x000F
#define REVERSE_ORDER      0x0F00
#define DEF_VAX            0
#define DESC_LEN           390
#define PROC_LEN           512

/*                            byte       number    contents
                             position   of bytes
                             in file
*/

struct header
    {
    int2   major_version;       /* 0-1          2  MAJOR_VER             */
    int2   minor_version;       /* 2-3          2  MINOR_VER             */
    int2   bits_per_pixel;      /* 4,5          2  packing factor        */
    char   operator[20];        /* 6-25        20  operator              */
    int2   chipnum;             /* 26,27        2  for multi-chip fpa    */
    int4   ext_hdr_len;         /* 28-31        4  len of post frame dat */
    char   filler[28];          /* 32-59       28  filler (unused)       */
    char   start_time[23];      /* 60-82       23  mm/dd/yy hh:mm:ss gmt */
    char   end_time[23];        /* 83-105      23  mm/dd/yy hh:mm:ss gmt */
    int2   row_start;           /* 106,107      2  row start position    */
    int2   row_length;          /* 108,109      2  # of cols (total/frm) */
    int2   col_start;           /* 110,111      2  col start position    */
    int2   col_length;          /* 112,113      2  # of rows (total/frm) */
    int2   frames_requested;    /* 114,115      2  # of frames requested */
    int2   frames_good;         /* 116,117      2  # of actual frames    */
    int2   big_little;          /* 118,119      2  Byte ordering         */
    int2   frame_type;          /* 120,121      2  frame type            */
    char   description[DESC_LEN]; /* 122,511  390  text description      */
    char   camera_status[512];  /* 512,1023   512  camera status         */
    char   utility_status[512]; /* 1024,1535  512  utility status        */
    char   los_status[512];     /* 1536,2047  512  telescope mount status*/
    char   proc_hist[PROC_LEN]; /* 2048,2559  512  processing history    */
    char   free[512];           /* 2560,3071  512  unused                */
    char   task_record[1024];   /* 3072,4095 1024  task message copy (binary)*/
  };  /* image header */

/*******************************/

/* Minimum required information: The following MUST be filled in:

   row_length: must contain the number of columns
   col_length: must contain the number of rows
   frames_good: must contain the number of frames in the file

   Defaults:

     frame_type: 0 = STANDARD_2BYTE_INT. Standard ETS data
     big_little: 0 = VAX ordering
*/

/************************ frame_type definitions *****************/

/* STANDARD_2BYTE_INT represents standard 2 byte, 12 bits per pixel ETS
     data with frames packed contiquously after the file header (no individual
     frame headers, etc.).
   FOURIER_AMP represents double precision floating point complex data wherein
     the real part is stored first followed by the imaginary part.
   ONE_BYTE_DATA is single byte (char) per pixel data
   FLOAT_DATA is floating point (4 bytes/pixel) data
*/

#define STANDARD_2BYTE_INT     0
#define FOURIER_AMP            2
#define ONE_BYTE_DATA          3
#define FLOAT_DATA             4

/***************** big_little byte order definitions ****************/

/*
     VAX & PCs (Intel 80x86 chips) use low byte first ordering
     Most RISC & Motorola systems (SUN, SGI, etc.) use high byte first ordering
     Data is written with 0x000F (15) in the big_little field. Thus the
     machine reading the data should swap/not swap bytes so that this field
     always reads 15, whether 00 0F or 0F 00.
     Most existing data is in VAX format with 0 in the big_little field.
*/

/*******************************/

typedef struct fileinfo
    {
      struct header file_header;
      FILE *dat_file;
      char filename[80];
      int current_frame;  /* base 1 */
      int this_machine;
      int data_order;
    } ETS_STRUCT;

typedef struct ir_annot_1
  {
    int frame_seq_num;
    double fp_temp;
    int gain;
    double bias;
    double intg_time;
    int mjd;
    int msec_time;
  } IR_ANNOT_1;

/******************** I/O function definitions *******************/

/** Bozos without ANSI C function prototyping need this kludge **/
#ifdef sun
#define NON_ANSI
#endif
#ifdef NON_ANSI
void fileopen();
int2 * readframe();
int2 * read_slbd_frame();
void filecreate();
void writeframe();
void fileclose();
void showhead();
FFTDAT * read_fft_frame();
void write_fft_frame();
int1 * read_int1_frame();
void write_int1_frame();
float * read_float_frame();
void write_float_frame();
void decode_ir();
void decode_IRIG_time();
#else

void fileopen
  (
   ETS_STRUCT * datafile
  );

int2 * readframe
  (
   ETS_STRUCT * datafile,
   int frame_num   /* if 0, then next frame, else get given frame_num */
  );

void slbdopen
  (
   ETS_STRUCT * datafile
  );

int2 * read_slbd_frame
  (
   ETS_STRUCT * datafile,
   int frame_num   /* if 0, then next frame, else get given frame_num */
  );

void filecreate
  (
   ETS_STRUCT * datafile
  );

void writeframe
  (
   ETS_STRUCT * datafile,      /* header structure to save image with */
   int2 * pix_array            /* 2-D array for image pixel data */
  );

void fileclose
  (
   FILE * dat_file
  );

void slbdclose
  (
   FILE * dat_file
  );

void showhead
  (
   ETS_STRUCT * datafile
  );

FFTDAT * read_fft_frame
  (
   ETS_STRUCT * datafile,
   int frame_num          /* if 0, then next frame, else get given frame # */
  );

void write_fft_frame
  (
   ETS_STRUCT * datafile,        /* header structure to save image with */
   FFTDAT * pix_array            /* 2-D array for image pixel data */
  );

int1 * read_int1_frame
  (
   ETS_STRUCT * datafile,
   int frame_num          /* if 0, then next frame, else get given frame # */
  );
void write_int1_frame
  (
   ETS_STRUCT * datafile,        /* header structure to save image with */
   int1 * pix_array              /* 2-D array for image pixel data */
  );

float * read_float_frame
  (
   ETS_STRUCT * datafile,
   int frame_num          /* if 0, then next frame, else get given frame # */
  );
void write_float_frame
  (
   ETS_STRUCT * datafile,        /* header structure to save image with */
   float * pix_array             /* 2-D array for image pixel data */
  );

void decode_ir
  (
   int nrows,
   int ncols,
   int2 * pix_array,
   IR_ANNOT_1 * annot_rtrn
   );

void decode_IRIG_time
  (
   char * irig_time,
   int * mjd,
   int * msec_time
   );
#endif
